from django.shortcuts import redirect
from django.core.urlresolvers import reverse
from django.conf import settings
from wagtail.wagtailcore.models import Site
from wagtail.contrib.settings.context_processors import SettingsProxy


class PersonaMiddleware(object):

    def process_request(self, request):
        site = Site.objects.get(is_default_site=True)
        setting = SettingsProxy(site)
        persona_settings = setting['usermetadata']['PersonaeSettings']

        if not persona_settings.persona_required:
            return None

        exclude = [
            settings.MEDIA_URL,
            settings.STATIC_URL,
            reverse('molo.usermetadata:persona'),
            reverse('health'),
            reverse('versions'),
            '/admin/',
            'django-admin/',
            '/import/',
            '/locale/'
        ]

        if hasattr(settings, 'PERSONA_IGNORE_PATH'):
            exclude += settings.PERSONA_IGNORE_PATH

        if any([p for p in exclude if request.path.startswith(p)]):
            return None

        if 'MOLO_PERSONA_SELECTION' not in request.session:
            url = '%s?next=%s' % (reverse('molo.usermetadata:persona'),
                                  request.path)
            return redirect(url)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import molo.core.models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailimages', '0012_copy_image_permissions_to_collections'),
        ('wagtailcore', '0028_merge'),
    ]

    operations = [
        migrations.CreateModel(
            name='PersonaIndexPage',
            fields=[
                ('page_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='PersonaPage',
            fields=[
                ('page_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('image', models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='wagtailimages.Image', null=True)),
            ],
            options={
                'verbose_name': 'Persona',
            },
            bases=(molo.core.models.TranslatablePageMixin, 'wagtailcore.page'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0028_merge'),
        ('usermetadata', '0002_create_index_page'),
    ]

    operations = [
        migrations.CreateModel(
            name='PersonaeSettings',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('persona_required', models.BooleanField(default=False, verbose_name='Activate Personae')),
                ('site', models.OneToOneField(editable=False, to='wagtailcore.Site')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterModelOptions(
            name='personapage',
            options={'verbose_name': 'Persona', 'verbose_name_plural': 'Personae'},
        ),
    ]



import pytest
from django.test import TestCase, Client
from django.core.urlresolvers import reverse

from molo.core.tests.base import MoloTestCaseMixin
from molo.core.models import SiteLanguage

from wagtail.wagtailcore.models import Site
from wagtail.contrib.settings.context_processors import SettingsProxy

from molo.usermetadata.models import PersonaIndexPage, PersonaPage


@pytest.mark.django_db
class TestPages(TestCase, MoloTestCaseMixin):

    def setUp(self):
        self.english = SiteLanguage.objects.create(locale='en')
        self.mk_main()

        self.index = PersonaIndexPage(title='Personae', slug="personae")
        self.main.add_child(instance=self.index)
        self.index.save_revision().publish()

        self.page = PersonaPage(title="child", slug="child")
        self.index.add_child(instance=self.page)
        self.page.save_revision().publish()

        self.client = Client()
        # Login
        self.user = self.login()

        site = Site.objects.get(is_default_site=True)
        setting = SettingsProxy(site)
        self.persona_settings = setting['usermetadata']['PersonaeSettings']
        self.persona_settings.persona_required = True
        self.persona_settings.save()

        self.site_settings = setting['core']['SiteSettings']
        self.site_settings.ga_tag_manager = 'GTM-xxxx'
        self.site_settings.save()

    def test_persona_selected_tag(self):

        response = self.client.get('/')
        self.assertRedirects(
            response, reverse('molo.usermetadata:persona') + '?next=/')

        response = self.client.get('%s?next=%s' % ((
            reverse(
                'molo.usermetadata:set_persona',
                kwargs={'persona_id': self.page.pk})),
            '/'))

        self.assertTrue(self.client.session['MOLO_PERSONA_SELECTED'])

        response = self.client.get('/')
        self.assertContains(response, 'persona=child')



from molo.usermetadata import views

from django.conf.urls import patterns, url


urlpatterns = patterns(
    '',
    url(r'^persona/$', views.PersonaView.as_view(), name='persona'),
    url(
        r'^persona/skip/$',
        views.skip_persona, name='skip_persona'),
    url(
        r'^persona/(?P<persona_id>\d+)/$',
        views.set_persona, name='set_persona'),
)




from django.db import models
from django.utils.translation import ugettext_lazy as _

from wagtail.wagtailcore.models import Page
from wagtail.wagtailadmin.edit_handlers import FieldPanel
from wagtail.wagtailimages.edit_handlers import ImageChooserPanel
from wagtail.contrib.settings.models import BaseSetting, register_setting

from molo.core.models import TranslatablePageMixin


@register_setting
class PersonaeSettings(BaseSetting):
    persona_required = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Activate Personae"),
    )


class PersonaIndexPage(Page):
    parent_page_types = []
    subpage_types = ['PersonaPage']


class PersonaPage(TranslatablePageMixin, Page):
    image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )

    class Meta:
        verbose_name = _('Persona')
        verbose_name_plural = _('Personae')

PersonaPage.content_panels = [
    FieldPanel('title', classname='full title'),
    ImageChooserPanel('image'),
]



import pytest
from django.test import TestCase, Client
from django.core.urlresolvers import reverse

from molo.core.tests.base import MoloTestCaseMixin
from molo.core.models import SiteLanguage

from wagtail.wagtailcore.models import Site
from wagtail.contrib.settings.context_processors import SettingsProxy

from molo.usermetadata.models import PersonaIndexPage, PersonaPage


@pytest.mark.django_db
class TestPages(TestCase, MoloTestCaseMixin):

    def setUp(self):
        self.english = SiteLanguage.objects.create(locale='en')
        self.french = SiteLanguage.objects.create(locale='fr')
        self.mk_main()

        self.index = PersonaIndexPage(title='Personae', slug="personae")
        self.main.add_child(instance=self.index)
        self.index.save_revision().publish()

        self.page = PersonaPage(title="child", slug="child")
        self.index.add_child(instance=self.page)
        self.page.save_revision().publish()
        self.page2 = PersonaPage(title="adult", slug="adult")
        self.index.add_child(instance=self.page2)
        self.page2.save_revision().publish()

        self.yourmind = self.mk_section(
            self.section_index, title='Your mind')
        self.yourmind_sub = self.mk_section(
            self.yourmind, title='Your mind subsection')

        self.client = Client()
        # Login
        self.user = self.login()

        site = Site.objects.get(is_default_site=True)
        setting = SettingsProxy(site)
        self.persona_settings = setting['usermetadata']['PersonaeSettings']
        self.persona_settings.persona_required = True
        self.persona_settings.save()

    def test_persona_page_is_deactivated(self):

        self.persona_settings.persona_required = False
        self.persona_settings.save()
        response = self.client.get('/')
        self.assertEquals(response.status_code, 200)

    def test_persona_page_redirect(self):

        response = self.client.get('/')
        self.assertRedirects(
            response, reverse('molo.usermetadata:persona') + '?next=/')

    def test_persona_page_redirect_from_section(self):

        response = self.client.get('/sections/your-mind/')
        self.assertRedirects(
            response, reverse(
                'molo.usermetadata:persona') + '?next=/sections/your-mind/')

    def test_translated_persona_page(self):
        self.client.post(reverse(
            'add_translation', args=[self.page.id, 'fr']))
        translated_page = PersonaPage.objects.get(
            slug='french-translation-of-child')
        translated_page.save_revision().publish()
        self.client.get('/locale/fr/')
        response = self.client.get(reverse('molo.usermetadata:persona'))
        self.assertContains(
            response, 'French translation of child')

    def test_set_persona_page(self):

        response = self.client.get('/')
        self.assertRedirects(
            response, reverse('molo.usermetadata:persona') + '?next=/')

        response = self.client.get('%s?next=%s' % ((
            reverse(
                'molo.usermetadata:set_persona',
                kwargs={'persona_id': self.page.pk})),
            '/'))
        self.assertRedirects(response, '/')

        response = self.client.get('/')
        self.assertEquals(response.status_code, 200)

    def test_skip_persona_page(self):

        response = self.client.get('/')
        self.assertRedirects(
            response, reverse('molo.usermetadata:persona') + '?next=/')

        response = self.client.get('%s?next=%s' % ((
            reverse('molo.usermetadata:skip_persona')), '/'))
        self.assertRedirects(response, '/')

        response = self.client.get('/')
        self.assertEquals(response.status_code, 200)

    def test_settings_persona_ignore_path(self):
        self.mk_articles(self.yourmind_sub)
        excl = ['/sections/your-mind/your-mind-subsection/test-page-1/']
        with self.settings(PERSONA_IGNORE_PATH=excl):
            response = self.client.get(excl[0])
            self.assertEquals(response.status_code, 200)



from django import template

register = template.Library()


@register.simple_tag(takes_context=True)
def persona_selected(context):
    request = context['request']
    if 'MOLO_PERSONA_SELECTED' in request.session:
        selected_persona = request.session['MOLO_PERSONA_SELECTION']
        del request.session['MOLO_PERSONA_SELECTED']
        return selected_persona
    return None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def create_persona_index(apps, schema_editor):
    from molo.core.models import Main
    from molo.usermetadata.models import PersonaIndexPage
    main = Main.objects.all().first()

    if main:
        persona_index = PersonaIndexPage(title='Personae', slug='Personae')
        main.add_child(instance=persona_index)
        persona_index.save_revision().publish()


class Migration(migrations.Migration):

    dependencies = [
        ('usermetadata', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(create_persona_index),
    ]



from django.views.generic import TemplateView
from django.shortcuts import get_object_or_404
from django.http import HttpResponseRedirect
from django.utils.translation import get_language_from_request
from molo.core.utils import get_locale_code
from molo.usermetadata.models import PersonaPage


class PersonaView(TemplateView):
    template_name = 'persona.html'

    def get_context_data(self, *args, **kwargs):
        persona_pages = PersonaPage.objects.live().filter(
            languages__language__is_main_language=True)
        context = super(PersonaView, self).get_context_data(*args, **kwargs)
        locale_code = get_locale_code(get_language_from_request(self.request))
        context.update({
            'persona_pages':
            [a.get_translation_for(locale_code) or a for a in persona_pages],
            'next': self.request.GET.get('next', '/')})
        return context


def set_persona(request, persona_id):
    persona = get_object_or_404(PersonaPage, pk=persona_id)
    request.session['MOLO_PERSONA_SELECTION'] = persona.slug
    request.session['MOLO_PERSONA_SELECTED'] = True
    return HttpResponseRedirect(request.GET.get('next', '/'))


def skip_persona(request):
    request.session['MOLO_PERSONA_SELECTION'] = 'skip'
    return HttpResponseRedirect(request.GET.get('next', '/'))
